@@ -5,30 +5,31 @@ Command line interface for ms3.
 """
 
 import argparse, os
+from datetime import datetime
 from typing import Optional
 
 import pandas as pd
 
-from ms3 import Score, Parse
-from ms3.operations import extract, check, compare, update, store_scores
-from ms3.utils import assert_dfs_equal, convert, convert_folder, get_musescore, resolve_dir, scan_directory, write_tsv, MS3_VERSION
+from ms3 import Parse
+from ms3.operations import extract, check, compare, update, store_scores, insert_labels_into_score
+from ms3.utils import convert_folder, resolve_dir, write_tsv, MS3_VERSION
 from ms3.logger import get_logger, inspect_loggers
 
 __author__ = "johentsch"
-__copyright__ = "Êcole Polytechnique Fédérale de Lausanne"
+__copyright__ = "École Polytechnique Fédérale de Lausanne"
 __license__ = "gpl3"
 
 def gather_extract_params(args):
     params = [name for name, arg in zip(
         ('measures', 'notes', 'rests', 'labels', 'expanded', 'form_labels', 'events', 'chords', 'metadata'),
-        (args.measures, args.notes, args.rests, args.labels, args.expanded, args.events, args.chords, args.metadata, args.form_labels))
-              if arg is not None]
+        (args.measures, args.notes, args.rests, args.labels, args.expanded, args.form_labels, args.events, args.chords, args.metadata))
+              if arg]
     return params
 
 
 
 
-def make_suffixes(args):
+def make_suffixes(args, include_metadata: bool = False):
     params = gather_extract_params(args)
     if args.suffix is None:
         suffixes = {}
@@ -40,29 +41,31 @@ def make_suffixes(args):
             suffixes = {f"{p}_suffix": args.suffix[0] for p in params}
         else:
             suffixes = {f"{p}_suffix": args.suffix[i] if i < l_suff else f"_{p}" for i, p in enumerate(params)}
-    if "metadata_suffix" in suffixes:
+    if not include_metadata and "metadata_suffix" in suffixes:
         del (suffixes["metadata_suffix"])
     return suffixes
 
 
-def add(args):
-    logger_cfg = {
-        'level': args.level,
-        'path': args.log,
-    }
-    p = Parse(args.dir, recursive=not args.nonrecursive, file_re=args.regex, exclude_re=args.exclude, paths=args.file, **logger_cfg)
-    p.parse(parallel=False)
-    if args.replace:
-        p.detach_labels()
-        p.logger.info(
-            f"Overview of the removed labels:\n{p.count_annotation_layers(which='detached').to_string()}")
-    p.add_labels(use=args.use)
-    ids = [id for id, score in p._parsed_mscx.items() if score.mscx.changed]
-    if args.out is not None:
-        p.store_scores(ids=ids, root_dir=args.out, overwrite=True)
+def add_cmd(args,
+            parse_obj: Optional[Parse] = None) -> Parse:
+    logger = get_logger('ms3.add', level=args.level)
+    if parse_obj is None:
+        p = make_parse_obj(args)
     else:
-        p.store_scores(ids=ids, overwrite=True)
+        p = parse_obj
 
+    insert_labels_into_score(p,
+                             facet=args.use,
+                             ask_for_input=args.ask,
+                             replace=args.replace,
+                             )
+    corpus2paths = store_scores(p,
+                                root_dir=args.out,
+                                folder='.',
+                                simulate=args.test,
+                                suffix=args.suffix)
+    changed = sum(map(len, corpus2paths.values()))
+    logger.info(f"Operation resulted in {changed} annotated score{'s' if changed != 1 else ''}.")
 
 def check_cmd(args,
               parse_obj: Optional[Parse] = None) -> Parse:
@@ -81,8 +84,6 @@ def compare_cmd(args,
                 parse_obj=None,
                 output: bool = True,
                 logger = None) -> None:
-    if logger is None:
-        logger = get_logger("ms3.compare", level=args.level)
     if parse_obj is None:
         p = make_parse_obj(args)
     else:
@@ -92,7 +93,7 @@ def compare_cmd(args,
                                  ask=args.ask,
                                  revision_specifier=args.compare,
                                  flip=args.flip,
-                                 logger=logger
+                                 logger=p.logger
                                  )
     logger.debug(f"{n_changed} files changed labels during comparison, {n_unchanged} didn't.")
     if output:
@@ -159,7 +160,7 @@ def extract_cmd(args, parse_obj: Optional[Parse] = None):
         p = parse_obj
     params = gather_extract_params(args)
     if len(params) == 0:
-        print("In order to extract DataFrames, pass at least one of the following arguments: -M (measures), -N (notes), -R (rests), -L (labels), -X (expanded), -E (events), -C (chords), -D (metadata) -F (form_labels)")
+        print("In order to extract DataFrames, pass at least one of the following arguments: -M (measures), -N (notes), -R (rests), -L (labels), -X (expanded), -F (form_labels), -E (events), -C (chords), -D (metadata)")
         return
     suffixes = make_suffixes(args)
     silence_label_warnings = args.silence_label_warnings if hasattr(args, 'silence_label_warnings') else False
@@ -218,46 +219,34 @@ def repair(args):
     print(args.dir)
 
 
-def transform(args):
-    if args.out is None:
-        args.out = os.getcwd()
-    params = [name for name, arg in zip(
-        ('measures', 'notes', 'rests', 'labels', 'expanded', 'events', 'chords', 'metadata'),
-        (args.measures, args.notes, args.rests, args.labels, args.expanded, args.events, args.chords, args.metadata))
-              if arg]
+def transform(args, parse_obj: Optional[Parse] = None):
+    suffixes = make_suffixes(args, include_metadata=True)
+    if parse_obj is None:
+        p = make_parse_obj(args, parse_tsv=True)
+    else:
+        p = parse_obj
+    params = gather_extract_params(args)
+    print(params)
     if len(params) == 0:
         print(
-            "Pass at least one of the following arguments: -M (measures), -N (notes), -R (rests), -L (labels), -X (expanded), -E (events), -C (chords), -D (metadata)")
+            "Pass at least one of the following arguments: -M (measures), -N (notes), -R (rests), -L (labels), -X (expanded), -F (form_labels), -E (events), -C (chords), -D (metadata)")
         return
-    if args.suffix is None:
-        suffixes = {f"{p}_suffix": '' for p in params}
-    else:
-        l_suff = len(args.suffix)
-        if l_suff == 0:
-            suffixes = {f"{p}_suffix": f"_{p}" for p in params}
-        elif l_suff == 1:
-            suffixes = {f"{p}_suffix": args.suffix[0] for p in params}
-        else:
-            suffixes = {f"{p}_suffix": args.suffix[i] if i < l_suff else f"_{p}" for i, p in enumerate(params)}
 
-    logger_cfg = {
-        'level': args.level,
-        'path': args.log,
-    }
-
-    p = Parse(args.dir, recursive=not args.nonrecursive, file_re=args.regex, exclude_re=args.exclude, paths=args.file, **logger_cfg)
-    p.parse_tsv()
     for param in params:
-        if param == 'metadata':
-            continue
-        sfx = suffixes[f"{param}_suffix"]
+        sfx = suffixes.get(f"{param}_suffix", '')
         tsv_name = f"concatenated_{param}{sfx}.tsv"
-        path = os.path.join(args.out, tsv_name)
+        if args.out is None:
+            path = tsv_name
+        else:
+            path = os.path.join(args.out, tsv_name)
+        if param == 'metadata':
+            df = p.metadata()
+        else:
+            df = p.get_facets(param, flat=True, interval_index=args.interval_index, unfold=args.unfold)
+        df = df.reset_index(drop=False)
         if args.test:
             p.logger.info(f"Would have written {path}.")
         else:
-            df = p.__getattribute__(param)(interval_index=args.interval_index, unfold=args.unfold)
-            df = df.reset_index(drop=False)
             write_tsv(df, path)
             p.logger.info(f"{path} written.")
 
@@ -306,21 +295,32 @@ def review_cmd(args,
                parse_obj: Optional[Parse] = None,
                ) -> None:
     """This command combines the functionalities check-extract-compare and additionally colors non-chord tones."""
-    logger = get_logger('ms3.review')
+    logger = get_logger('ms3.review', level=args.level)
     if parse_obj is None:
         p = make_parse_obj(args)
     else:
         p = parse_obj
+
+    # create file for warnings
+    out_dir = '.' if args.out is None else args.out
+    warnings_file = os.path.join(out_dir, 'warnings.log')
+    header = f"Warnings encountered during the last execution of ms3 review (v{MS3_VERSION})"
+    header = f"{header}\n{'=' * len(header)}\n\n"
+    with open(warnings_file, 'w', encoding='utf-8') as f:
+        f.write(header)
+
+    # call ms3 check
     if args.ignore_scores + args.ignore_labels < 2:
         what = '' if args.ignore_scores else 'SCORES'
         if args.ignore_labels:
             what += 'LABELS' if what == '' else 'AND LABELS'
         print(f"CHECKING {what}...")
         test_passes = check(p,
-              ignore_labels=args.ignore_labels,
-              ignore_scores=args.ignore_scores,
-              assertion=False,
-              parallel=False)
+                            ignore_labels=args.ignore_labels,
+                            ignore_scores=args.ignore_scores,
+                            assertion=False,
+                            parallel=False,
+                            warnings_file=warnings_file)
         p.parse_tsv()
     else:
         p.parse(parallel=False)
@@ -330,12 +330,17 @@ def review_cmd(args,
             msg += "\nI was disregarding all files whose file names are not listed in the column 'fname' of a 'metadata.tsv' file. " \
                    "Add -a if you want me to include all scores."
         print(msg)
+        os.remove(warnings_file)
         return
+
+    # call ms3 extract
     params = gather_extract_params(args)
     if len(params) > 0:
         print("EXTRACTING FACETS...")
         extract_cmd(args, p)
-    print("COLORING NON-CHORD TONES...")
+
+    # color out-of-label tones
+    print("COLORING OUT-OF-LABEL TONES...")
     review_reports = p.color_non_chord_tones()
     if len(review_reports) > 0:
         dataframes, keys = [], []
@@ -350,8 +355,15 @@ def review_cmd(args,
         if warning_selection.sum() > 0:
             test_passes = False
             filtered_report = report[warning_selection]
-            logger.warning(filtered_report.to_string(columns=['mc', 'mn', 'mc_onset', 'label', 'chord_tones', 'added_tones', 'n_colored', 'n_untouched', 'count_ratio']),
+            pretty_report = filtered_report.to_string(columns=['mc', 'mn', 'mc_onset', 'label', 'chord_tones', 'added_tones', 'n_colored', 'n_untouched', 'count_ratio'])
+            logger.warning(pretty_report,
                            extra={'message_id': (19,)})
+            with open(warnings_file, 'a', encoding='utf-8') as f:
+                f.write(f"Chord labels where the ratio of colored notes vs. all notes in a segment lies above {args.threshold}:\n")
+                f.write(pretty_report)
+            logger.info(f"Added the {warning_selection.sum()} labels to {warnings_file}.")
+
+    # call ms3 compare
     if args.compare is not None:
         revision = None if args.compare == '' else args.compare
         commit_str = '' if revision is None else f" @ {revision}"
@@ -371,9 +383,10 @@ def review_cmd(args,
     changed = sum(map(len, corpus2paths.values()))
     logger.info(f"Operation resulted in {changed} review file{'s' if changed != 1 else ''}.")
     if test_passes:
-        logger.info(f"Parsed scores passed all tests.")
+        logger.info(f"Parsed scores passed all tests. Removed {warnings_file}")
+        os.remove(warnings_file)
     else:
-        msg = "Not all tests have passed."
+        msg = f"Not all tests have passed. Please check the relevant warnings in {warnings_file}"
         if args.fail:
             assert test_passes, msg
         else:
@@ -550,9 +563,11 @@ The library offers you the following commands. Add the flag -h to one of them to
     #                            help="Remove labels from selected staves only. 1=upper staff; -1=lowest staff (default)")
     # add_parser.add_argument('--type', default=1,
     #                            help="Only remove particular types of harmony labels.")
+    add_parser.add_argument('-s', '--suffix', metavar='SUFFIX', default='_annotated',
+                                help='Suffix of the new scores with inserted labels. Defaults to _annotated.')
     add_parser.add_argument('--replace', action='store_true',
                                help="Remove existing labels from the scores prior to adding. Like calling ms3 empty first.")
-    add_parser.set_defaults(func=add)
+    add_parser.set_defaults(func=add_cmd)
 
 
 
@@ -630,29 +645,32 @@ In particular, check DCML harmony labels for syntactic correctness.""", parents=
     review_parser.set_defaults(func=review_cmd)
 
     transform_parser = subparsers.add_parser('transform',
-                                          help="Concatenate and transform TSV data from one or several corpora.",
+                                          help="Concatenate and transform TSV data from one or several corpora. "
+                                               "Available transformations are unfolding repeats and adding an interval index.",
                                           parents=[parse_args])
     transform_parser.add_argument('-M', '--measures', action='store_true',
-                                help="Folder where to store TSV files with measure information needed for tasks such as unfolding repetitions.")
+                                help="Concatenate measures TSVs for all selected pieces.")
     transform_parser.add_argument('-N', '--notes', action='store_true',
-                                help="Folder where to store TSV files with information on all notes.")
+                                help="Concatenate notes TSVs for all selected pieces.")
     transform_parser.add_argument('-R', '--rests', action='store_true',
-                                help="Folder where to store TSV files with information on all rests.")
+                                help="Concatenate rests TSVs for all selected pieces (use ms3 extract -R to create those).")
     transform_parser.add_argument('-L', '--labels', action='store_true',
-                                help="Folder where to store TSV files with information on all annotation labels.")
+                                help="Concatenate raw harmony label TSVs for all selected pieces (use ms3 extract -L to create those).")
     transform_parser.add_argument('-X', '--expanded', action='store_true',
-                                help="Folder where to store TSV files with expanded DCML labels.")
+                                help="Concatenate expanded DCML label TSVs for all selected pieces.")
+    transform_parser.add_argument('-F', '--form_labels', metavar='folder', nargs='?', const='../form_labels',
+                              help="Concatenate form label TSVs for all selected pieces.")
     transform_parser.add_argument('-E', '--events', action='store_true',
-                                help="Folder where to store TSV files with all events (notes, rests, articulation, etc.) without further processing.")
+                                help="Concatenate events TSVs (notes, rests, articulation, etc.) for all selected pieces (use ms3 extract -E to create those).")
     transform_parser.add_argument('-C', '--chords', action='store_true',
-                                help="Folder where to store TSV files with <chord> tags, i.e. groups of notes in the same voice with identical onset and duration. The tables include lyrics, slurs, and other markup.")
+                                help="Concatenate harmony chords TSVs (lyrics, slurs, and other markup) for all selected pieces (use ms3 extract -C to create those).")
     transform_parser.add_argument('-D', '--metadata', action='store_true',
-                                help="Directory or full path for storing one TSV file with metadata. If no filename is included in the path, it is called metadata.tsv")
+                                help="Output 'concatenated_metadata.tsv' with one row per selected piece.")
     transform_parser.add_argument('-s', '--suffix', nargs='*', metavar='SUFFIX',
                                 help="Pass -s to use standard suffixes or -s SUFFIX to choose your own. In the latter case they will be assigned to the extracted aspects in the order "
                                      "in which they are listed above (capital letter arguments).")
     transform_parser.add_argument('-u', '--unfold', action='store_true',
-                                help="Unfold the repeats for all stored DataFrames.")
+                                help="Unfold the repeats for all concatenated DataFrames.")
     transform_parser.add_argument('--interval_index', action='store_true',
                               help="Prepend a column with [start, end) intervals to the TSV files.")
     transform_parser.set_defaults(func=transform)
